class Canvas {
	svg;
	_offsetX = 0;
	_offsetY = 0;
	append(element) {
		this.svg.append(element)
	};
	correctViewBox() {
		this.svg.setAttribute('viewBox', `${-this.width / 2 - this._offsetX} ${-this.height / 2 + this._offsetY} ${this.width} ${this.height}`)
	}
	constructor(svg) {
		this.svg = svg;
		Object.defineProperties(this, {
			width: {
				get() {
					return Number(this.svg.getAttribute('width'))
				},
				set(width) {
					this.svg.setAttribute('width', width)
					this.correctViewBox()
				},
			},
			height: {
				get() {
					return Number(this.svg.getAttribute('height'))
				},
				set(height) {
					this.svg.setAttribute('height', height)
					this.correctViewBox()
				},
			},
			offsetX: {
				get() {
					return this._offsetX
				},
				set(offset) {
					this._offsetX = offset
					this.correctViewBox()
				}
			},
			offsetY: {
				get() {
					return this._offsetY
				},
				set(offset) {
					this._offsetY = offset
					this.correctViewBox()
				}
			}
		})
		this.correctViewBox()

		let copy = this
		this.svg.onmousedown = function(event) {
			// Выйти, если перемещают объект
			if (event.target !== event.currentTarget)
				return

			this.style.cursor = 'grabbing'

			let initialPageX = event.pageX
			let initialPageY = event.pageY

			let initialOffsetX = copy._offsetX
			let initialOffsetY = copy._offsetY

			document.onmousemove = (event) => {
				copy.offsetX = initialOffsetX + event.pageX - initialPageX
				copy.offsetY = initialOffsetY + initialPageY - event.pageY
			}

			this.onmouseup = () => {
				document.onmousemove = null
				this.style.cursor = ''
			}
		}
	}
}

var CurrentCanvas;

function getCurrentCanvas() {
	return CurrentCanvas
}

function setCurrentCanvas(canvas) {
	CurrentCanvas = canvas	
}

export { Canvas, getCurrentCanvas, setCurrentCanvas }
