import { getCurrentCanvas } from './canvas.js'
import { GeoObject } from './geo.js'

class _Dot extends GeoObject {
	constructor(properties = {}) {
		super();
		let circle = document.createElementNS('http://www.w3.org/2000/svg', 'circle')
		this.svg = circle;
		Object.defineProperties(this, {
			x: {
				get() {
					return Number(this.svg.getAttribute('cx'))
				},
				set(x) {
					this.svg.setAttribute('cx', x)
					this.calcBind()
				}
			},
			y: {
				get() {
					return -Number(this.svg.getAttribute('cy'))
				},
				set(y) {
					this.svg.setAttribute('cy', -y)
					this.calcBind()
				}
			},
			borderColor: {
				get() {
					return this.svg.getAttribute('stroke')
				},
				set(color) {
					this.svg.setAttribute('stroke', color)
				}
			},
			borderSize: {
				get() {
					return Number(this.svg.getAttribute('stroke-width'))
				},
				set(size) {
					this.svg.setAttribute('stroke-width', size)
				}
			},
			color: {
				get() {
					return this.svg.getAttribute('fill')
				},
				set(color) {
					this.svg.setAttribute('fill', color)
				}
			},
			size: {
				get() {
					return Number(this.svg.getAttribute('r')) * 2
				},
				set(size) {
					this.svg.setAttribute('r', size / 2)
				}
			}
		})
		for (let p of ['borderColor', 'borderSize', 'color', 'draggable', 'size']) {
			if (properties[p])
				this[p] = properties[p]
			else if (window.Style.Dot[p])
				this[p] = window.Style.Dot[p]
		}
		let copy = this
		this.svg.style.cursor = 'grab'
		this.svg.onmousedown = function(event) { 
			if (! copy.draggable)
				return
			this.style.cursor = 'grabbing'
			let initialPageX = event.pageX
			let initialPageY = event.pageY

			let initialX = copy.x
			let initialY = copy.y

			document.onmousemove = function(event) {
				copy.x = initialX + event.pageX - initialPageX
				copy.y = initialY + initialPageY - event.pageY
			}

			this.onmouseup = function() {
				document.onmousemove = null
				this.style.cursor = 'grab'
			}
		}
	}
}

class Dot extends _Dot {
	constructor(x = 0, y = 0, properties = {}) {
		super(properties);
		this.x = x;
		this.y = y;
		getCurrentCanvas().append(this.svg)
	}
}

class Middle extends _Dot {
	d1; d2;
	constructor(d1, d2, properties = {}) {
		super(properties)
		this.d1 = d1;
		this.d2 = d2;
		d1.bind.push(this)
		d2.bind.push(this)
		this.calc()
		getCurrentCanvas().append(this.svg)
	}
	calc() {
		this.x = (this.d1.x + this.d2.x) / 2
		this.y = (this.d1.y + this.d2.y) / 2
	}
}

export { Dot, Middle }
